from tqdm import tqdm
from sklearn.neighbors import KNeighborsRegressor
import numpy as np
import xarray as xr
import logging


def analogs_seas_hcst(exp,src,target):
    downscaled = None
    for looyear in tqdm(exp.sdate.dt.year):

        X_fcst = exp.isel(sdate=exp.sdate.dt.year==looyear)

        X_train = src.sel(time=np.logical_not(src.time.isin(X_fcst.leadtimes)))
        y_train = target.sel(time=np.logical_not(target.time.isin(X_fcst.leadtimes)))

        y_fcst = analogs_fit_and_predict(X_fcst,X_train,y_train,15)
        
        if downscaled is None:
            downscaled = y_fcst
        else:
            downscaled = xr.concat([downscaled, y_fcst], dim="sdate")
    
    downscaled.coords["leadtimes"] = exp.leadtimes

    #downscaled = downscaled.drop_dims("vdate")
    downscaled = downscaled.to_dataset()
        
    #downscaled.attrs = exp.attrs
    return(downscaled)

def analogs_seas_fcst(exp,src,target):
    
    downscaled = analogs_fit_and_predict(exp,src,target,15).to_dataset()
    
    downscaled = downscaled.assign_coords(sdate=exp.sdate)
        
    return(downscaled)

def analogs_fit_and_predict(X_fcst,X_train,y_train,n_knns):
       
    # Stack multidimensional arrays for sklearn
    X_train = X_train.stack(feature=("lon","lat"))
    y_train = y_train.stack(pos=("lon","lat")).dropna(dim='pos',how='any')
    X_fcst = X_fcst.stack(sample=("sdate","time","ensemble")).stack(feature=("lon","lat"))

    knn = KNeighborsRegressor(n_neighbors=n_knns)
    y_fcst = knn.fit(X_train,y_train).predict(X_fcst)

    # reassign coords and unstack output
    y_fcst = xr.DataArray(y_fcst, dims=["sample","pos"], 
                          coords=[X_fcst.coords['sample'], 
                                  y_train.coords['pos']]).unstack()
    #y_fcst.coords["vdate"] = (["sdate", "time"], y_fcst.coords["sdate"]+y_fcst.coords["time"])
    y_fcst.name = y_train.name

    return(y_fcst)
    
